package tokyo.swet.uitest;

import android.support.test.runner.AndroidJUnit4;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;

import tokyo.swet.uitest.helper.ActivityTestBase;

import static android.support.test.espresso.Espresso.onView;
import static android.support.test.espresso.action.ViewActions.click;
import static android.support.test.espresso.matcher.ViewMatchers.assertThat;
import static android.support.test.espresso.matcher.ViewMatchers.hasErrorText;
import static android.support.test.espresso.matcher.ViewMatchers.isDisplayed;
import static android.support.test.espresso.matcher.ViewMatchers.withId;
import static android.support.test.espresso.matcher.ViewMatchers.withText;
import static tokyo.swet.uitest.helper.AssetHelper.testUser;
import static tokyo.swet.uitest.helper.UserHelper.logout;
import static tokyo.swet.uitest.helper.UserHelper.registerUser;
import static tokyo.swet.uitest.helper.UserHelper.temporaryMailAddress;
import static tokyo.swet.uitest.helper.UserHelper.temporaryNickname;
import static tokyo.swet.uitest.helper.UserHelper.temporaryPassword;
import static tokyo.swet.uitest.helper.ViewHelper.waitUntilDisplayed;
import static tokyo.swet.uitest.helper.ViewHelper.waitUntilTextDisplayed;

/**
 * ユーザ登録に関するテスト
 * Created by SWET.
 */

@RunWith(AndroidJUnit4.class)
public class UserRegisterTests extends ActivityTestBase {

    @Before
    public void setup() {
        logout();
        onView(withId(R.id.auth_button_openRegister)).perform(click());
    }

    @After
    public void teardown() {
    }

    @Test
    public void registerNewUser() {
        // 各入力欄用の値を用意
        final String nickname = temporaryNickname();
        final String email    = temporaryMailAddress();
        final String password = temporaryPassword();

        // 登録フォームに値を入れ登録ボタンをクリック
        registerUser(nickname, email, password);

        // ニックネームが表示されるまで待つ
        waitUntilDisplayed(R.id.auth_label_nickname);

        // 登録したニックネームと
        // メールアドレスが表示されていることの確認
        assertThat(
                "登録したニックネームが表示されている",
                activity.findViewById(R.id.auth_label_nickname),
                withText(nickname)
        );
        assertThat(
                "登録したメールアドレスが表示されている",
                activity.findViewById(R.id.auth_label_email),
                withText(email)
        );

        // ログアウトボタンが表示されていることの確認
        assertThat(
                "ログアウトボタンが表示されている",
                activity.findViewById(R.id.auth_button_logout),
                isDisplayed()
        );
    }

    @Test
    public void registerUserWithEmpty() {
        // 明示的に入力欄をすべて空文字にして登録
        registerUser("", "", "");

        // 各入力欄にエラーメッセージが
        // 表示されていることの確認
        String expectMessage = messageHelper.geLocalizedMessage(R.string.error_required);
        assertThat(
                "正しいメッセージが表示されている",
                activity.findViewById(R.id.auth_input_registerNickname),
                hasErrorText(expectMessage)
        );
        assertThat(
                "正しいメッセージが表示されている",
                activity.findViewById(R.id.auth_input_registerEmail),
                hasErrorText(expectMessage)
        );
        assertThat(
                "正しいメッセージが表示されている",
                activity.findViewById(R.id.auth_input_registerPassword),
                hasErrorText(expectMessage)
        );
    }

    @Test
    public void registerUserWithAlreadyRegisteredEmail() {
        // 登録済みユーザーのメールアドレスを使用
        registerUser(temporaryNickname(), testUser().getEmail(), temporaryPassword());

        // エラーメッセージが表示されるまで待つ
        waitUntilTextDisplayed(R.id.auth_label_registerErrors);

        // エラーメッセージが表示されていることの確認
        String expectMessage = messageHelper.geLocalizedMessage(R.string.error_used_email);
        assertThat(
                "正しいメッセージが表示されている",
                activity.findViewById(R.id.auth_label_registerErrors),
                withText(expectMessage)
        );
    }
}
