package tokyo.swet.uitest.helper;

import android.support.test.espresso.Espresso;
import android.support.test.espresso.NoMatchingViewException;
import android.support.test.espresso.PerformException;
import android.view.View;
import android.widget.TextView;

import junit.framework.AssertionFailedError;

import org.hamcrest.Matcher;

import tokyo.swet.uitest.MainActivity;
import tokyo.swet.uitest.R;
import tokyo.swet.uitest.helper.idling.IdlingResourceText;
import tokyo.swet.uitest.helper.idling.IdlingResourceVisible;

import static android.support.test.espresso.Espresso.onView;
import static android.support.test.espresso.action.ViewActions.click;
import static android.support.test.espresso.action.ViewActions.swipeLeft;
import static android.support.test.espresso.assertion.ViewAssertions.matches;
import static android.support.test.espresso.matcher.ViewMatchers.isDisplayed;
import static android.support.test.espresso.matcher.ViewMatchers.withId;
import static tokyo.swet.uitest.matcher.UITestViewMatcher.toMatcher;

/**
 * Created by SWET.
 */

public class ViewHelper {

    public static MainActivity mActivity;

    /**
     * 指定したタブまで移動する
     * @param name タブ
     */
    public static void selectTab(Matcher<View> name) {
        waitUntilDisplayed(R.id.pager_tab_strip);
        for (int mPageTitle : MainActivity.pageTitles) {
            try {
                onView(name).perform(click());
                return;
            } catch (NoMatchingViewException | AssertionFailedError | PerformException ignore) {
            }
            onView(withId(R.id.pager_tab_strip)).check(matches(isDisplayed())).perform(swipeLeft());
        }
        throw new IllegalArgumentException("そのような要素はありません: " + name);
    }

    /**
     * テキストが表示されるまで待ちます
     * @param view TextView
     */
    public static void waitUntilTextDisplayed(TextView view) {
        IdlingResourceText resource = new IdlingResourceText(view);
        try {
            Espresso.registerIdlingResources(resource);
            onView(toMatcher(view)).check(matches(isDisplayed()));
        } finally {
            Espresso.unregisterIdlingResources(resource);
        }
    }

    /**
     * テキストが表示されるまで待ちます
     * @param resourceId Resource ID
     */
    public static void waitUntilTextDisplayed(int resourceId) {
        waitUntilTextDisplayed((TextView) mActivity.findViewById(resourceId));
    }

    /**
     * 要素が表示されるまで待ちます
     * @param view View
     */
    public static void waitUntilDisplayed(View view) {
        IdlingResourceVisible resource = new IdlingResourceVisible(view);
        try {
            Espresso.registerIdlingResources(resource);
            onView(toMatcher(view)).check(matches(isDisplayed()));
        } finally {
            Espresso.unregisterIdlingResources(resource);
        }
    }

    /**
     * 要素が表示されるまで待ちます
     * @param resourceId Resource ID
     */
    public static void waitUntilDisplayed(int resourceId) {
        waitUntilDisplayed(mActivity.findViewById(resourceId));
    }
}
