package tokyo.swet.uitest.matcher;

import android.support.design.widget.TextInputLayout;
import android.support.test.espresso.matcher.BoundedMatcher;
import android.view.View;

import org.hamcrest.Description;
import org.hamcrest.Matcher;
import org.hamcrest.TypeSafeMatcher;

import static android.support.test.internal.util.Checks.checkNotNull;
import static org.hamcrest.core.Is.is;

/**
 * Created by SWET.
 */

public class UITestViewMatcher {

    /**
     * View を Matcher に変換します
     * @param v View
     * @return a TypeSageMatcher
     */
    public static Matcher<View> toMatcher(final View v) {
        return new TypeSafeMatcher<View>() {
            @Override
            protected boolean matchesSafely(View item) {
                return item == v;
            }

            @Override

            public void describeTo(Description description) {
                description.appendText(v.toString());
            }
        };
    }

    /**
     * 指定された Matcher と {@link TextInputLayout} の持つエラーメッセージが一致するか判定する Matcher を返します。
     *
     * @param stringMatcher
     * @return Matcher
     */
    public static Matcher<View> hasErrorTextInLayout(final Matcher<String> stringMatcher) {
        checkNotNull(stringMatcher);
        return new BoundedMatcher<View, TextInputLayout>(TextInputLayout.class) {

            @Override
            public void describeTo(Description description) {
                description.appendText("with error: ");
                stringMatcher.describeTo(description);
            }

            @Override
            protected boolean matchesSafely(TextInputLayout view) {
                return stringMatcher.matches(view.getError().toString());
            }
        };
    }

    /**
     * 指定されたエラーメッセージが含まれているか判定する Matcher を返します
     *
     * @param expectedError 期待するエラーメッセージ
     * @return Matcher
     */
    public static Matcher<View> hasErrorTextInLayout(final String expectedError) {
        return hasErrorTextInLayout(is(expectedError));
    }
}
