require './spec/spec_helper'

describe "既存ユーザーのシナリオ", type: :registered_user do
  before do
    # 登録済みアカウントでログインする
    login_as_registered_user
  end

  context "ログインしてノート機能を利用する" do
    before do
      # ノートページに移動する
      click_note_tab
    end

    # すでに複数のノートがあることを確認する
    it "すでに複数のノートがある" do
      expect(note_titles.size).to be >= 1
    end

    context "新規にノートを作成する" do
      let(:title) { random_title }
      let(:content) { random_content }

      before do
        # 新規にノートを作成する
        create_note(title: title, content: content)
      end

      # ノートが登録されたことを確認する
      it "ノートが一覧に表示されている" do
        expect(latest_note_title).to eq title
      end
    end
  end
end

describe "新規ユーザーのシナリオ", type: :new_user do
  before do
    logout
  end

  context "ユーザー登録前にノート機能を利用する" do
    before do
      # 未ログイン状態でノートページに移動する
      click_note_tab
    end

    # エラーメッセージが表示される
    it "エラーメッセージが表示されている" do
      expect(error_unauthorized_message).not_to be_nil
    end

    context "ユーザー登録してノート機能を利用する" do
      context "ユーザー登録を行う" do
        before do
          # ユーザー登録を行いログイン状態になる
          register_user
          click_note_tab
        end

        # ノートが1件も登録されていないことを確認する
        it "ノートが1件もない" do
          expect(note_titles.size).to be == 0
        end

        context "ノート機能を利用する" do
          let(:title) { random_title }
          let(:content) { random_content }

          # 新規にノートを作成する
          before do
            click_note_tab
            create_note(title: title, content: content)
          end

          # ノートが登録されたことを確認する
          it "ノートが一覧に表示されている" do
            expect(latest_note_title).to eq title
          end
        end
      end
    end
  end
end


