require 'randexp-multibyte'

module NoteHelper
  include ElementHelper

  # 日本語を含むランダムなタイトルを取得する
  # @return [String] タイトル
  def random_title
    "タイトル: #{/[:japanese:]{10}/.gen}"
  end

  # 日本語を含むランダムな内容を取得する
  # @return [String] 内容
  def random_content
    "内容: #{/[:japanese:]{10}/.gen}"
  end

  # タイトルと内容を指定して新しいノートを作成する
  # @param [String] title タイトル
  # @param [String] content 内容
  # @return [void]
  def create_note(title:, content:)
    click_note_tab

    click_button(id: 'notes_button_create')
    click_button(id: 'note_button_clear')

    input_text(id: 'note_input_title', text: title)
    input_text(id: 'note_input_content', text: content)

    click_button(id: 'note_button_save')
  end

  # ノートのタイトル一覧を取得する
  # @return [[Element]] ノートのタイトル一覧
  def note_titles
    click_note_tab

    note_titles_list
  end

  # 最新のノートのタイトルを取得する
  # @return [String]
  def latest_note_title
    note_titles.first.text
  end

  # ノートタブを開く
  # @return [void]
  def click_note_tab
    click_tab(name: 'Note')
  end
end
