require 'securerandom'

module UserHelper
  include ElementHelper

  # ランダムなニックネームを取得する
  # @return [String] ニックネーム
  def temporary_nickname
    "SWET#{SecureRandom.hex(5)}"
  end

  # ランダムなE-mailを取得する
  # @return [String] E-mail
  def temporary_email
    "#{SecureRandom.hex(6)}@example.com"
  end

  # ラムダムなパスワードを取得する
  # @return [String] パスワード
  def temporary_password
    SecureRandom.hex(10)
  end

  # ユーザを新規登録する
  # @param email    メールアドレス
  # @param password パスワード
  # @param nickname ニックネーム
  # @return [void]
  def register_user(email: temporary_email, password: temporary_password, nickname: temporary_nickname)
    click_user_tab

    click_button(id: 'auth_button_openRegister')

    input_text(id: 'auth_input_registerNickname', text: nickname)
    input_text(id: 'auth_input_registerEmail', text: email)
    input_text(id: 'auth_input_registerPassword', text: password)

    click_button(id: 'auth_button_register')

    wait_true {
      exists { find_element(*condition(id: 'auth_button_logout')) }
    }
  end

  # 既に登録されているユーザでログインする
  # @return [void]
  def login_as_registered_user
    logout
    login(email: user['email'], password: user['password'])
  end

  # メールアドレスとパスワードを指定してログインする
  # @param [String] email メールアドレス
  # @param [String] password パスワード
  # @return [void]
  def login(email:, password:)
    click_user_tab

    input_text(id: 'auth_input_loginEmail', text: email)
    input_text(id: 'auth_input_loginPassword', text: password)

    click_button(id: 'auth_button_login')

    wait_true {
      exists { find_element(*condition(id: 'auth_button_logout')) }
    }
  end

  # ログアウトする
  # @return [void]
  def logout
    click_user_tab
    click_button(id: 'auth_button_logout')

    wait_true {
      exists { find_element(*condition(id: 'auth_button_login')) }
    }
  end

  # 未ログインエラーのラベルを取得する
  # @return [String] エラー文字列
  def error_unauthorized_message
    label(id: 'notes_label_unauthorized')
  end

  # ユーザタブを開く
  # @return [void]
  def click_user_tab
    click_tab(name: 'User')
  end
end
