//
// Copyright (c) 2017 SWET. All rights reserved.
//

import UIKit
import Firebase

class LoginController: UIViewController, UITextFieldDelegate {
    
    @IBOutlet weak var email: UITextField!
    @IBOutlet weak var emailError: UILabel!
    
    @IBOutlet weak var password: UITextField!
    @IBOutlet weak var passwordError: UILabel!
    
    @IBOutlet weak var errorMessages: UILabel!

    @IBOutlet weak var loginButton: UIButton!
    @IBOutlet weak var registerButton: UIButton!

    override func viewDidLoad() {
        super.viewDidLoad()

        self.email.delegate = self
        self.password.delegate = self

        self.configureLocalize()
        self.clearErrors()
    }
    
    // MARK: - Action

    @IBAction func tapLogin(_ sender: Any) {
        self.view.endEditing(true)
        self.clearErrors()
        self.login()
    }

    @IBAction func openLogin(_ segue: UIStoryboardSegue) {
    }

    // MARK: - UITextFieldDelegate
    
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        textField.resignFirstResponder()
        return true
    }
    
    // MARK: - Private

    /// ボタンなどのローカライズ
    private func configureLocalize() {
        self.email.placeholder = NSLocalizedString("email", comment: "メールアドレス")
        self.password.placeholder = NSLocalizedString("password", comment: "パスワード")
        self.loginButton.setTitle(NSLocalizedString("login", comment: "ログイン"), for: .normal)
        self.registerButton.setTitle(NSLocalizedString("register", comment: "登録"), for: .normal)
    }

    /// 入力されている値でログインを試行する
    private func login() {
        
        guard self.validInput() else { return }

        guard
            let email = self.email.text,
            let password = self.password.text else { return }

        
        self.clearErrors()

        self.showNetworkActivityIndicator()

        Auth.auth().signIn(withEmail: email, password: password) { [weak self] (user, error) in

            self?.hideNetworkActivityIndicator()

            // Login Error
            if let error = error {
                self?.errorMessages.text = Auth.localizedErrorMessage(from: error, for: .login)
                return
            }

            // Login Success
            if let _ = user {
                self?.clearErrors()
            }
        }
    }

    /// 入力された値を検証する
    ///
    /// - Returns: 検証に成功したかどうか
    private func validInput() -> Bool {
        
        var valid = true
        
        // Email
        switch self.email.validate(rule: EmailValidationRule()) {
        case .invalid(let errors):
            emailError.text = errors.first?.message
            valid = false
        case .valid:
            break
        }
        
        // Password
        switch self.password.validate(rule: PasswordValidationRule()) {
        case .invalid(let errors):
            passwordError.text = errors.first?.message
            valid = false
        case .valid:
            break
        }
        
        return valid
    }

    /// エラー表示をクリアする
    private func clearErrors() {
        self.emailError.clear()
        self.passwordError.clear()
        self.errorMessages.clear()
    }
}
