//
//  Copyright (c) 2017 SWET. All rights reserved.
//

import XCTest

class Assets {

    static var registeredUser: User {
        guard let path = Bundle(for: Assets.self).path(forResource: "data", ofType: "plist") else {
            XCTFail("リソースの読み込みに失敗しました")
            return User(nickname: "", email: "", password: "")
        }
        guard
            let userData = NSDictionary(contentsOfFile: path)?.value(forKey: "testuser") as? NSDictionary,
            let nickname = userData.value(forKey: "nickname") as? String,
            let email = userData.value(forKey: "email") as? String,
            let password = userData.value(forKey: "password") as? String
        else {
            XCTFail("テストユーザデータの読み込みに失敗しました")
            return User(nickname: "", email: "", password: "")
        }
        return User(nickname: nickname, email: email, password: password)
    }

}

/// Unixtimeを取得します
///
/// - Returns: unixtime
func unixtime() -> TimeInterval {
    return Date().timeIntervalSince1970
}

/// ユニークなニックネームを取得します
///
/// - Returns: ニックネーム文字列
func temporaryNickname() -> String {
    return "nickname.\(arc4random())"
}

/// ユニークなメールアドレスを取得します
///
/// - Returns: メールアドレス文字列
func temporaryMailAddress() -> String {

    // e.g. "swet+1492835838.3133@example.com"
    return "swet+\(unixtime())@example.com"
}

/// ランダムなパスワードを取得します
///
/// - Returns: パスワード文字列
func temporaryPassword() -> String {
    return "\(arc4random())"
}

/// ランダムなタイトルを取得します
///
/// - Returns: 文字列
func randomTitle() -> String {
    return "タイトル: \(unixtime())"
}

/// ランダムなコンテンツを取得します
///
/// - Returns: 文字列
func randomContent() -> String {
    return "内容: \(unixtime())"
}

class Localized {

    func string(forKey key: String) -> String {
        let languageCode = NSLocale.current.languageCode

        guard
            let languageBundlePath = Bundle(for: type(of: self)).path(forResource: languageCode, ofType: "lproj"),
            let localizationBundle = Bundle(path: languageBundlePath)
        else {
            XCTFail("リソースの読み込みに失敗しました")
            return ""
        }
        return NSLocalizedString(key, bundle: localizationBundle, comment: "")
    }

}
