//
// Copyright (c) 2017 SWET. All rights reserved.
//

import XCTest

/// ログインに関するテスト
class UserLoginTests: XCTestCase {

    let app = XCUIApplication()

    override func setUp() {
        super.setUp()
        continueAfterFailure = false
        app.launch()

        logout()
    }

    override func tearDown() {
        super.tearDown()
    }

    func testLoginAsRegisteredAccount() {
        // 登録済みユーザでログイン
        loginAsRegisteredUser()

        // 登録したニックネームと
        // メールアドレスが表示されていることの確認
        XCTAssertEqual(UserStatus.nicknameLabel.label, RegisteredUserNickname, "登録したニックネームが表示されている")
        XCTAssertEqual(UserStatus.emailLabel.label, RegisteredUserEmail, "登録したメールアドレスがが表示されている")

        // ログアウトボタンが表示されていることの確認
        XCTAssertTrue(UserStatus.logoutButton.isHittable, "ログアウトボタンが表示されている")
    }

    func testLoginWithInvalidPassword() {
        // 存在するユーザーの
        // メールアドレスと不適切なパスワードを入力する
        login(email: RegisteredUserEmail, password: "invalid_password")

        // エラーメッセージが表示されるまで最大10秒待つ
        waitHittable(for: Login.errorMessageLabel)

        // エラーメッセージが表示されていることの確認
        let expectMessage = Localized().string(forKey: "error_login_wrongPassword")
        XCTAssertEqual(Login.errorMessageLabel.label, expectMessage, "正しいメッセージが表示されている")
    }

}
