//
// Copyright (c) 2017 SWET. All rights reserved.
//

import XCTest

/// ユーザ登録に関するテスト
class UserRegisterTests: XCTestCase {

    let app = XCUIApplication()

    override func setUp() {
        super.setUp()
        continueAfterFailure = false
        app.launch()

        logout()
    }

    override func tearDown() {
        super.tearDown()
    }

    func testRegisterUser() {
        // 各入力欄用の値を用意
        let nickname = temporaryNickname()
        let email    = temporaryMailAddress()
        let password = temporaryPassword()

        registerUser(nickname: nickname, email: email, password: password)

        // ログアウトボタンが表示されるまで最大10秒待つ
        waitExists(for: UserStatus.logoutButton)

        // 登録したニックネームと
        // メールアドレスが表示されていることの確認
        XCTAssertEqual(UserStatus.nicknameLabel.label, nickname, "登録したニックネームが表示されている")
        XCTAssertEqual(UserStatus.emailLabel.label, email, "登録したメールアドレスがが表示されている")

        // ログアウトボタンが表示されていることの確認
        XCTAssertTrue(UserStatus.logoutButton.isHittable, "ログアウトボタンが表示されている")
    }

    func testRegisterUserWithEmpty() {
        // 登録フォームに何も値を入れずに登録
        registerUser(nickname: "", email: "", password: "")

        // 各入力欄にエラーメッセージが表示されていることの確認
        let expectMessage = Localized().string(forKey: "error_required")
        XCTAssertEqual(UserRegister.nicknameErrorLabel.label, expectMessage, "正しいメッセージが表示されている")
        XCTAssertEqual(UserRegister.emailErrorLabel.label, expectMessage, "正しいメッセージが表示されている")
        XCTAssertEqual(UserRegister.passwordErrorLabel.label, expectMessage, "正しいメッセージが表示されている")
    }

    func testRegisterUserWithAlreadyRegisteredEmail() {
        // 登録済みユーザーのメールアドレスを使用
        registerUser(nickname: temporaryNickname(),
                     email: RegisteredUserEmail,
                     password: temporaryPassword())

        // エラーメッセージが表示されるまで最大10秒待つ
        waitHittable(for: UserRegister.errorMessageLabel)

        // エラーメッセージが表示されていることの確認
        let expectMessage = Localized().string(forKey: "error_used_email")
        XCTAssertEqual(UserRegister.errorMessageLabel.label, expectMessage, "正しいメッセージが表示されている")
    }

}
